@@ -17,6 +17,7 @@
  */
 package org.apache.hadoop.hdds.scm;
 
+import java.io.Closeable;
 import java.io.IOException;
 
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
@@ -24,7 +25,7 @@ import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 /**
  * Interface to provide XceiverClient when needed.
  */
-public interface XceiverClientFactory {
+public interface XceiverClientFactory extends Closeable {
 
   XceiverClientSpi acquireClient(Pipeline pipeline) throws IOException;
 
