@@ -103,6 +103,13 @@ public interface NetworkTopology {
    */
   int getNumOfNodes(int level);
 
+  /**
+   * Return the nodes at level <i>level</i>.
+   * @param level topology level, start from 1, which means ROOT
+   * @return the nodes on the level
+   */
+  List<Node> getNodes(int level);
+
   /**
    * Randomly choose a node in the scope.
    * @param scope range of nodes from which a node will be chosen. If scope
@@ -176,7 +183,8 @@ public interface NetworkTopology {
    * @return the chosen node
    */
   Node chooseRandom(String scope, List<String>  excludedScopes,
-      Collection<Node> excludedNodes, Node affinityNode, int ancestorGen);
+      Collection<? extends Node> excludedNodes, Node affinityNode,
+      int ancestorGen);
 
   /**
    * Choose the node at index <i>index</i> from <i>scope</i>, share the same
