@@ -20,6 +20,7 @@ package org.apache.hadoop.ozone.container.keyvalue.helpers;
 
 import java.io.IOException;
 
+import org.apache.hadoop.hdds.client.BlockID;
 import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 import org.apache.hadoop.hdds.scm.container.common.helpers.StorageContainerException;
@@ -27,6 +28,7 @@ import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.utils.ContainerCache;
 import org.apache.hadoop.ozone.container.common.utils.ReferenceCountedDB;
+import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainer;
 import org.apache.hadoop.ozone.container.keyvalue.KeyValueContainerData;
 
 import com.google.common.base.Preconditions;
@@ -36,6 +38,8 @@ import org.apache.hadoop.ozone.container.metadata.DatanodeStoreSchemaTwoImpl;
 
 import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.NO_SUCH_BLOCK;
 import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.UNABLE_TO_READ_METADATA_DB;
+import static org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos.Result.UNKNOWN_BCSID;
+import static org.apache.hadoop.ozone.container.common.utils.StorageVolumeUtil.onFailure;
 
 /**
  * Utils functions to help block functions.
@@ -103,7 +107,7 @@ public final class BlockUtils {
    *
    * @param containerData containerData.
    * @param conf configuration.
-   * @return MetadataStore handle.
+   * @return DB handle.
    * @throws StorageContainerException
    */
   public static ReferenceCountedDB getDB(KeyValueContainerData containerData,
@@ -118,6 +122,7 @@ public final class BlockUtils {
           .getContainerDBType(), containerData.getDbFile().getAbsolutePath(),
               containerData.getSchemaVersion(), conf);
     } catch (IOException ex) {
+      onFailure(containerData.getVolume());
       String message = String.format("Error opening DB. Container:%s " +
           "ContainerPath:%s", containerData.getContainerID(), containerData
           .getDbFile().getPath());
@@ -179,5 +184,27 @@ public final class BlockUtils {
     }
   }
 
+  /**
+   * Verify if request block BCSID is supported.
+   *
+   * @param container container object.
+   * @param blockID requested block info
+   * @throws IOException if cannot support block's blockCommitSequenceId
+   */
+  public static void verifyBCSId(KeyValueContainer container, BlockID blockID)
+      throws IOException {
+    long bcsId = blockID.getBlockCommitSequenceId();
+    Preconditions.checkNotNull(blockID,
+        "BlockID cannot be null");
+    Preconditions.checkNotNull(container,
+        "Container cannot be null");
 
-}
\ No newline at end of file
+    long containerBCSId = container.getBlockCommitSequenceId();
+    if (containerBCSId < bcsId) {
+      throw new StorageContainerException(
+          "Unable to find the block with bcsID " + bcsId + " .Container "
+              + container.getContainerData().getContainerID() + " bcsId is "
+              + containerBCSId + ".", UNKNOWN_BCSID);
+    }
+  }
+}
