@@ -26,11 +26,15 @@ import org.apache.hadoop.hdds.protocol.datanode.proto.ContainerProtos;
 
 import org.apache.hadoop.hdds.scm.container.common.helpers
     .StorageContainerException;
+import org.apache.hadoop.hdds.utils.db.DBProfile;
+import org.apache.hadoop.hdds.utils.db.RDBStore;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.helpers.BlockData;
 import org.apache.hadoop.ozone.container.common.impl.ChunkLayOutVersion;
 import org.apache.hadoop.ozone.container.common.impl.ContainerDataYaml;
+import org.apache.hadoop.ozone.container.common.utils.StorageVolumeUtil;
+import org.apache.hadoop.ozone.container.common.utils.db.DatanodeDBProfile;
 import org.apache.hadoop.ozone.container.common.volume.HddsVolume;
 import org.apache.hadoop.ozone.container.common.volume
     .RoundRobinVolumeChoosingPolicy;
@@ -38,7 +42,8 @@ import org.apache.hadoop.ozone.container.common.volume.VolumeSet;
 import org.apache.hadoop.ozone.container.common.volume.MutableVolumeSet;
 import org.apache.hadoop.ozone.container.keyvalue.helpers.BlockUtils;
 import org.apache.hadoop.ozone.container.metadata.AbstractDatanodeStore;
-import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.hadoop.ozone.container.metadata.DatanodeStore;
+import org.apache.ozone.test.GenericTestUtils;
 import org.apache.hadoop.util.DiskChecker;
 import org.apache.hadoop.ozone.container.common.utils.ReferenceCountedDB;
 
@@ -51,7 +56,9 @@ import org.junit.rules.TemporaryFolder;
 import org.junit.runner.RunWith;
 import org.junit.runners.Parameterized;
 import org.mockito.Mockito;
+import org.rocksdb.ColumnFamilyHandle;
 import org.rocksdb.ColumnFamilyOptions;
+import org.rocksdb.RocksDBException;
 
 import java.io.File;
 
@@ -64,9 +71,11 @@ import java.util.Map;
 import java.util.List;
 import java.util.UUID;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.Supplier;
 import java.util.stream.Collectors;
 import java.util.stream.IntStream;
 
+import static org.apache.hadoop.hdds.HddsConfigKeys.HDDS_DB_PROFILE;
 import static org.apache.ratis.util.Preconditions.assertTrue;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertFalse;
@@ -175,8 +184,8 @@ public class TestKeyValueContainer {
             datanodeId.toString());
     KeyValueContainer container = new KeyValueContainer(containerData, CONF);
 
-    HddsVolume containerVolume = volumeChoosingPolicy.chooseVolume(volumeSet
-        .getVolumesList(), 1);
+    HddsVolume containerVolume = volumeChoosingPolicy.chooseVolume(
+        StorageVolumeUtil.getHddsVolumesList(volumeSet.getVolumesList()), 1);
     String hddsVolumeDir = containerVolume.getHddsRootDir().toString();
 
     container.populatePathFields(scmId, containerVolume, hddsVolumeDir);
@@ -206,8 +215,33 @@ public class TestKeyValueContainer {
       fail("Container is imported twice. Previous files are overwritten");
     } catch (IOException ex) {
       //all good
+      assertTrue(container.getContainerFile().exists());
     }
 
+    //Import failure should cleanup the container directory
+    containerData =
+        new KeyValueContainerData(containerId + 1,
+            keyValueContainerData.getLayOutVersion(),
+            keyValueContainerData.getMaxSize(), UUID.randomUUID().toString(),
+            datanodeId.toString());
+    container = new KeyValueContainer(containerData, CONF);
+
+    containerVolume = volumeChoosingPolicy.chooseVolume(
+        StorageVolumeUtil.getHddsVolumesList(volumeSet.getVolumesList()), 1);
+    hddsVolumeDir = containerVolume.getHddsRootDir().toString();
+    container.populatePathFields(scmId, containerVolume, hddsVolumeDir);
+    try {
+      FileInputStream fis = new FileInputStream(folderToExport);
+      fis.close();
+      container.importContainerData(fis, packer);
+      fail("Container import should fail");
+    } catch (Exception ex) {
+      assertTrue(ex instanceof IOException);
+    } finally {
+      File directory =
+          new File(container.getContainerData().getContainerPath());
+      assertFalse(directory.exists());
+    }
   }
 
   /**
@@ -405,30 +439,96 @@ public class TestKeyValueContainer {
   }
 
   @Test
-  public void testContainersShareColumnFamilyOptions() throws Exception {
-    // Get a read only view (not a copy) of the options cache.
-    Map<ConfigurationSource, ColumnFamilyOptions> cachedOptions =
-        AbstractDatanodeStore.getColumnFamilyOptionsCache();
+  public void testContainerRocksDB()
+      throws StorageContainerException, RocksDBException {
+    closeContainer();
+    keyValueContainer = new KeyValueContainer(
+        keyValueContainerData, CONF);
+    keyValueContainer.create(volumeSet, volumeChoosingPolicy, scmId);
 
+    try(ReferenceCountedDB db =
+        BlockUtils.getDB(keyValueContainerData, CONF)) {
+      RDBStore store = (RDBStore) db.getStore().getStore();
+      long defaultCacheSize = 64 * OzoneConsts.MB;
+      long cacheSize = Long.parseLong(store
+          .getProperty("rocksdb.block-cache-capacity"));
+      Assert.assertEquals(defaultCacheSize, cacheSize);
+      for (ColumnFamilyHandle handle : store.getColumnFamilyHandles()) {
+        cacheSize = Long.parseLong(
+            store.getProperty(handle, "rocksdb.block-cache-capacity"));
+        Assert.assertEquals(defaultCacheSize, cacheSize);
+      }
+    }
+  }
+
+  @Test
+  public void testContainersShareColumnFamilyOptions() {
+    ConfigurationSource conf = new OzoneConfiguration();
+
+    // Make sure ColumnFamilyOptions are same for a particular db profile
+    for (Supplier<DatanodeDBProfile> dbProfileSupplier : new Supplier[] {
+        DatanodeDBProfile.Disk::new, DatanodeDBProfile.SSD::new }) {
+      // ColumnFamilyOptions should be same across configurations
+      ColumnFamilyOptions columnFamilyOptions1 = dbProfileSupplier.get()
+          .getColumnFamilyOptions(new OzoneConfiguration());
+      ColumnFamilyOptions columnFamilyOptions2 = dbProfileSupplier.get()
+          .getColumnFamilyOptions(new OzoneConfiguration());
+      Assert.assertEquals(columnFamilyOptions1, columnFamilyOptions2);
+
+      // ColumnFamilyOptions should be same when queried multiple times
+      // for a particulat configuration
+      columnFamilyOptions1 = dbProfileSupplier.get()
+          .getColumnFamilyOptions(conf);
+      columnFamilyOptions2 = dbProfileSupplier.get()
+          .getColumnFamilyOptions(conf);
+      Assert.assertEquals(columnFamilyOptions1, columnFamilyOptions2);
+    }
+
+    // Make sure ColumnFamilyOptions are different for different db profile
+    DatanodeDBProfile diskProfile = new DatanodeDBProfile.Disk();
+    DatanodeDBProfile ssdProfile = new DatanodeDBProfile.SSD();
+    Assert.assertNotEquals(
+        diskProfile.getColumnFamilyOptions(new OzoneConfiguration()),
+        ssdProfile.getColumnFamilyOptions(new OzoneConfiguration()));
+    Assert.assertNotEquals(diskProfile.getColumnFamilyOptions(conf),
+        ssdProfile.getColumnFamilyOptions(conf));
+  }
+
+  @Test
+  public void testDBProfileAffectsDBOptions() throws Exception {
     // Create Container 1
     keyValueContainer.create(volumeSet, volumeChoosingPolicy, scmId);
-    Assert.assertEquals(1, cachedOptions.size());
-    ColumnFamilyOptions options1 = cachedOptions.get(CONF);
-    Assert.assertNotNull(options1);
 
-    // Create Container 2
+    DatanodeDBProfile outProfile1;
+    try (ReferenceCountedDB db1 =
+        BlockUtils.getDB(keyValueContainer.getContainerData(), CONF)) {
+      DatanodeStore store1 = db1.getStore();
+      Assert.assertTrue(store1 instanceof AbstractDatanodeStore);
+      outProfile1 = ((AbstractDatanodeStore) store1).getDbProfile();
+    }
+
+    // Create Container 2 with different DBProfile in otherConf
+    OzoneConfiguration otherConf = new OzoneConfiguration();
+    // Use a dedicated profile for test
+    otherConf.setEnum(HDDS_DB_PROFILE, DBProfile.SSD);
+
     keyValueContainerData = new KeyValueContainerData(2L,
         layout,
         (long) StorageUnit.GB.toBytes(5), UUID.randomUUID().toString(),
         datanodeId.toString());
-    keyValueContainer = new KeyValueContainer(keyValueContainerData, CONF);
+    keyValueContainer = new KeyValueContainer(keyValueContainerData, otherConf);
     keyValueContainer.create(volumeSet, volumeChoosingPolicy, scmId);
 
-    Assert.assertEquals(1, cachedOptions.size());
-    ColumnFamilyOptions options2 = cachedOptions.get(CONF);
-    Assert.assertNotNull(options2);
+    DatanodeDBProfile outProfile2;
+    try (ReferenceCountedDB db2 =
+        BlockUtils.getDB(keyValueContainer.getContainerData(), otherConf)) {
+      DatanodeStore store2 = db2.getStore();
+      Assert.assertTrue(store2 instanceof AbstractDatanodeStore);
+      outProfile2 = ((AbstractDatanodeStore) store2).getDbProfile();
+    }
 
-    // Column family options object should be reused.
-    Assert.assertSame(options1, options2);
+    // DBOtions should be different
+    Assert.assertNotEquals(outProfile1.getDBOptions().compactionReadaheadSize(),
+        outProfile2.getDBOptions().compactionReadaheadSize());
   }
 }
