@@ -29,16 +29,25 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ExecutorService;
 
 import org.apache.hadoop.hdds.HddsConfigKeys;
+import org.apache.hadoop.hdds.client.RatisReplicationConfig;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
+import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.SCMCommandProto;
 import org.apache.hadoop.hdds.scm.ScmConfigKeys;
 import org.apache.hadoop.hdds.scm.TestUtils;
+import org.apache.hadoop.hdds.scm.ha.MockSCMHAManager;
+import org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator;
+import org.apache.hadoop.hdds.scm.ha.SCMServiceManager;
+import org.apache.hadoop.hdds.scm.ha.SCMContext;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
 import org.apache.hadoop.hdds.scm.node.NodeStatus;
 import org.apache.hadoop.hdds.scm.container.CloseContainerEventHandler;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
+import org.apache.hadoop.hdds.scm.container.ContainerManagerImpl;
+import org.apache.hadoop.hdds.scm.container.ContainerManager;
 import org.apache.hadoop.hdds.scm.container.MockNodeManager;
-import org.apache.hadoop.hdds.scm.container.SCMContainerManager;
 import org.apache.hadoop.hdds.scm.container.common.helpers.AllocatedBlock;
 import org.apache.hadoop.hdds.scm.container.common.helpers.ExcludeList;
 import org.apache.hadoop.hdds.scm.events.SCMEvents;
@@ -48,8 +57,9 @@ import org.apache.hadoop.hdds.scm.pipeline.MockRatisPipelineProvider;
 import org.apache.hadoop.hdds.scm.pipeline.Pipeline;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineProvider;
-import org.apache.hadoop.hdds.scm.pipeline.SCMPipelineManager;
+import org.apache.hadoop.hdds.scm.pipeline.PipelineManagerImpl;
 import org.apache.hadoop.hdds.scm.safemode.SCMSafeModeManager;
+import org.apache.hadoop.hdds.scm.safemode.SCMSafeModeManager.SafeModeStatus;
 import org.apache.hadoop.hdds.scm.server.SCMConfigurator;
 import org.apache.hadoop.hdds.scm.server.StorageContainerManager;
 import org.apache.hadoop.hdds.server.events.EventHandler;
@@ -59,7 +69,7 @@ import org.apache.hadoop.ozone.OzoneConsts;
 import org.apache.hadoop.ozone.container.common.SCMTestUtils;
 import org.apache.hadoop.ozone.protocol.commands.CommandForDatanode;
 import org.apache.hadoop.ozone.protocol.commands.CreatePipelineCommand;
-import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils;
 
 import static org.apache.hadoop.ozone.OzoneConsts.GB;
 import static org.apache.hadoop.ozone.OzoneConsts.MB;
@@ -76,14 +86,16 @@ import org.junit.rules.TemporaryFolder;
  */
 public class TestBlockManager {
   private StorageContainerManager scm;
-  private SCMContainerManager mapping;
+  private ContainerManager mapping;
   private MockNodeManager nodeManager;
-  private SCMPipelineManager pipelineManager;
+  private PipelineManagerImpl pipelineManager;
   private BlockManagerImpl blockManager;
+  private SCMHAManager scmHAManager;
+  private SequenceIdGenerator sequenceIdGen;
   private static final long DEFAULT_BLOCK_SIZE = 128 * MB;
-  private HddsProtos.ReplicationFactor factor;
-  private HddsProtos.ReplicationType type;
   private EventQueue eventQueue;
+  private SCMContext scmContext;
+  private SCMServiceManager serviceManager;
   private int numContainerPerOwnerInPipeline;
   private OzoneConfiguration conf;
 
@@ -93,6 +105,7 @@ public class TestBlockManager {
   @Rule
   public TemporaryFolder folder= new TemporaryFolder();
   private SCMMetadataStore scmMetadataStore;
+  private ReplicationConfig replicationConfig;
 
   @Before
   public void setUp() throws Exception {
@@ -107,30 +120,45 @@ public class TestBlockManager {
     conf.setTimeDuration(HddsConfigKeys.HDDS_PIPELINE_REPORT_INTERVAL, 5,
         TimeUnit.SECONDS);
 
-    // Override the default Node Manager in SCM with this Mock Node Manager.
+    // Override the default Node Manager and SCMHAManager
+    // in SCM with the Mock one.
     nodeManager = new MockNodeManager(true, 10);
+    scmHAManager = MockSCMHAManager.getInstance(true);
+
     eventQueue = new EventQueue();
+    scmContext = SCMContext.emptyContext();
+    serviceManager = new SCMServiceManager();
 
     scmMetadataStore = new SCMMetadataStoreImpl(conf);
     scmMetadataStore.start(conf);
+
+    sequenceIdGen = new SequenceIdGenerator(
+        conf, scmHAManager, scmMetadataStore.getSequenceIdTable());
+
     pipelineManager =
-        new SCMPipelineManager(conf, nodeManager,
+        PipelineManagerImpl.newPipelineManager(
+            conf,
+            scmHAManager,
+            nodeManager,
             scmMetadataStore.getPipelineTable(),
-            eventQueue);
-    pipelineManager.allowPipelineCreation();
+            eventQueue,
+            scmContext,
+            serviceManager);
 
     PipelineProvider mockRatisProvider =
         new MockRatisPipelineProvider(nodeManager,
             pipelineManager.getStateManager(), conf, eventQueue);
     pipelineManager.setPipelineProvider(HddsProtos.ReplicationType.RATIS,
         mockRatisProvider);
-    SCMContainerManager containerManager =
-        new SCMContainerManager(conf,
-            scmMetadataStore.getContainerTable(),
-            scmMetadataStore.getStore(),
-            pipelineManager);
+    ContainerManager containerManager =
+        new ContainerManagerImpl(conf,
+            scmHAManager,
+            sequenceIdGen,
+            pipelineManager,
+            scmMetadataStore.getContainerTable());
     SCMSafeModeManager safeModeManager = new SCMSafeModeManager(conf,
-        containerManager.getContainers(), pipelineManager, eventQueue) {
+        containerManager.getContainers(), containerManager,
+        pipelineManager, eventQueue, serviceManager, scmContext) {
       @Override
       public void emitSafeModeStatus() {
         // skip
@@ -142,22 +170,21 @@ public class TestBlockManager {
     configurator.setContainerManager(containerManager);
     configurator.setScmSafeModeManager(safeModeManager);
     configurator.setMetadataStore(scmMetadataStore);
+    configurator.setSCMHAManager(scmHAManager);
+    configurator.setScmContext(scmContext);
     scm = TestUtils.getScm(conf, configurator);
 
     // Initialize these fields so that the tests can pass.
-    mapping = (SCMContainerManager) scm.getContainerManager();
+    mapping = scm.getContainerManager();
     blockManager = (BlockManagerImpl) scm.getScmBlockManager();
     DatanodeCommandHandler handler = new DatanodeCommandHandler();
     eventQueue.addHandler(SCMEvents.DATANODE_COMMAND, handler);
     CloseContainerEventHandler closeContainerHandler =
-        new CloseContainerEventHandler(pipelineManager, mapping);
+        new CloseContainerEventHandler(pipelineManager, mapping, scmContext);
     eventQueue.addHandler(SCMEvents.CLOSE_CONTAINER, closeContainerHandler);
-    factor = HddsProtos.ReplicationFactor.THREE;
-    type = HddsProtos.ReplicationType.RATIS;
+    replicationConfig = new RatisReplicationConfig(ReplicationFactor.THREE);
 
-
-    blockManager.onMessage(
-        new SCMSafeModeManager.SafeModeStatus(false, false), null);
+    scm.getScmContext().updateSafeModeStatus(new SafeModeStatus(false, true));
   }
 
   @After
@@ -170,10 +197,10 @@ public class TestBlockManager {
 
   @Test
   public void testAllocateBlock() throws Exception {
-    pipelineManager.createPipeline(type, factor);
+    pipelineManager.createPipeline(replicationConfig);
     TestUtils.openAllRatisPipelines(pipelineManager);
     AllocatedBlock block = blockManager.allocateBlock(DEFAULT_BLOCK_SIZE,
-        type, factor, OzoneConsts.OZONE, new ExcludeList());
+        replicationConfig, OzoneConsts.OZONE, new ExcludeList());
     Assert.assertNotNull(block);
   }
 
@@ -181,27 +208,28 @@ public class TestBlockManager {
   public void testAllocateBlockWithExclusion() throws Exception {
     try {
       while (true) {
-        pipelineManager.createPipeline(type, factor);
+        pipelineManager.createPipeline(replicationConfig);
       }
     } catch (IOException e) {
     }
     TestUtils.openAllRatisPipelines(pipelineManager);
     ExcludeList excludeList = new ExcludeList();
     excludeList
-        .addPipeline(pipelineManager.getPipelines(type, factor).get(0).getId());
+        .addPipeline(pipelineManager.getPipelines(replicationConfig)
+            .get(0).getId());
     AllocatedBlock block = blockManager
-        .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+        .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig, OzoneConsts.OZONE,
             excludeList);
     Assert.assertNotNull(block);
     for (PipelineID id : excludeList.getPipelineIds()) {
       Assert.assertNotEquals(block.getPipeline().getId(), id);
     }
 
-    for (Pipeline pipeline : pipelineManager.getPipelines(type, factor)) {
+    for (Pipeline pipeline : pipelineManager.getPipelines(replicationConfig)) {
       excludeList.addPipeline(pipeline.getId());
     }
     block = blockManager
-        .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+        .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig, OzoneConsts.OZONE,
             excludeList);
     Assert.assertNotNull(block);
     Assert.assertTrue(
@@ -223,7 +251,7 @@ public class TestBlockManager {
       CompletableFuture.supplyAsync(() -> {
         try {
           future.complete(blockManager
-              .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor,
+              .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
                   OzoneConsts.OZONE,
                   new ExcludeList()));
         } catch (IOException e) {
@@ -251,7 +279,7 @@ public class TestBlockManager {
     for (int i = 0; i < threadCount; i++) {
       executors.add(Executors.newSingleThreadExecutor());
     }
-    pipelineManager.createPipeline(type, factor);
+    pipelineManager.createPipeline(replicationConfig);
     TestUtils.openAllRatisPipelines(pipelineManager);
     Map<Long, List<AllocatedBlock>> allocatedBlockMap =
             new ConcurrentHashMap<>();
@@ -264,9 +292,9 @@ public class TestBlockManager {
         try {
           List<AllocatedBlock> blockList;
           AllocatedBlock block = blockManager
-                  .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor,
-                          OzoneConsts.OZONE,
-                          new ExcludeList());
+              .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
+                  OzoneConsts.OZONE,
+                  new ExcludeList());
           long containerId = block.getBlockID().getContainerID();
           if (!allocatedBlockMap.containsKey(containerId)) {
             blockList = new ArrayList<>();
@@ -287,7 +315,9 @@ public class TestBlockManager {
       CompletableFuture
               .allOf(futureList.toArray(
                       new CompletableFuture[futureList.size()])).get();
-      Assert.assertTrue(pipelineManager.getPipelines(type).size() == 1);
+
+      Assert.assertTrue(
+          pipelineManager.getPipelines(replicationConfig).size() == 1);
       Assert.assertTrue(
               allocatedBlockMap.size() == numContainerPerOwnerInPipeline);
       Assert.assertTrue(allocatedBlockMap.
@@ -311,7 +341,7 @@ public class TestBlockManager {
     for (int i = 0; i < threadCount; i++) {
       executors.add(Executors.newSingleThreadExecutor());
     }
-    pipelineManager.createPipeline(type, factor);
+    pipelineManager.createPipeline(replicationConfig);
     TestUtils.openAllRatisPipelines(pipelineManager);
     Map<Long, List<AllocatedBlock>> allocatedBlockMap =
             new ConcurrentHashMap<>();
@@ -324,9 +354,9 @@ public class TestBlockManager {
         try {
           List<AllocatedBlock> blockList;
           AllocatedBlock block = blockManager
-                  .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor,
-                          OzoneConsts.OZONE,
-                          new ExcludeList());
+              .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
+                  OzoneConsts.OZONE,
+                  new ExcludeList());
           long containerId = block.getBlockID().getContainerID();
           if (!allocatedBlockMap.containsKey(containerId)) {
             blockList = new ArrayList<>();
@@ -347,20 +377,21 @@ public class TestBlockManager {
       CompletableFuture
               .allOf(futureList.toArray(
                       new CompletableFuture[futureList.size()])).get();
-      Assert.assertTrue(
-              pipelineManager.getPipelines(type).size() == 1);
-      Pipeline pipeline = pipelineManager.getPipelines(type).get(0);
+      Assert.assertEquals(1,
+          pipelineManager.getPipelines(replicationConfig).size());
+      Pipeline pipeline =
+          pipelineManager.getPipelines(replicationConfig).get(0);
       // total no of containers to be created will be number of healthy
       // volumes * number of numContainerPerOwnerInPipeline which is equal to
       // the thread count
-      Assert.assertTrue(threadCount == pipelineManager.
+      Assert.assertEquals(threadCount, pipelineManager.
               getNumberOfContainers(pipeline.getId()));
-      Assert.assertTrue(
-              allocatedBlockMap.size() == threadCount);
-      Assert.assertTrue(allocatedBlockMap.
-              values().size() == threadCount);
+      Assert.assertEquals(threadCount,
+              allocatedBlockMap.size());
+      Assert.assertEquals(threadCount, allocatedBlockMap.
+              values().size());
       allocatedBlockMap.values().stream().forEach(v -> {
-        Assert.assertTrue(v.size() == 1);
+        Assert.assertEquals(1, v.size());
       });
     } catch (Exception e) {
       Assert.fail("testAllocateBlockInParallel failed");
@@ -378,7 +409,7 @@ public class TestBlockManager {
     for (int i = 0; i < threadCount; i++) {
       executors.add(Executors.newSingleThreadExecutor());
     }
-    pipelineManager.createPipeline(type, factor);
+    pipelineManager.createPipeline(replicationConfig);
     TestUtils.openAllRatisPipelines(pipelineManager);
     Map<Long, List<AllocatedBlock>> allocatedBlockMap =
         new ConcurrentHashMap<>();
@@ -391,7 +422,7 @@ public class TestBlockManager {
         try {
           List<AllocatedBlock> blockList;
           AllocatedBlock block = blockManager
-              .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor,
+              .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
                   OzoneConsts.OZONE,
                   new ExcludeList());
           long containerId = block.getBlockID().getContainerID();
@@ -415,8 +446,9 @@ public class TestBlockManager {
           .allOf(futureList.toArray(
               new CompletableFuture[futureList.size()])).get();
       Assert.assertTrue(
-          pipelineManager.getPipelines(type).size() == 1);
-      Pipeline pipeline = pipelineManager.getPipelines(type).get(0);
+          pipelineManager.getPipelines(replicationConfig).size() == 1);
+      Pipeline pipeline =
+          pipelineManager.getPipelines(replicationConfig).get(0);
       // the pipeline per raft log disk config is set to 1 by default
       int numContainers = (int)Math.ceil((double)
               (numContainerPerOwnerInPipeline *
@@ -437,44 +469,45 @@ public class TestBlockManager {
     long size = 6 * GB;
     thrown.expectMessage("Unsupported block size");
     blockManager.allocateBlock(size,
-        type, factor, OzoneConsts.OZONE, new ExcludeList());
+        replicationConfig, OzoneConsts.OZONE, new ExcludeList());
   }
 
 
   @Test
   public void testAllocateBlockFailureInSafeMode() throws Exception {
-    blockManager.onMessage(
-        new SCMSafeModeManager.SafeModeStatus(true, true), null);
+    scm.getScmContext().updateSafeModeStatus(
+        new SCMSafeModeManager.SafeModeStatus(true, true));
     // Test1: In safe mode expect an SCMException.
     thrown.expectMessage("SafeModePrecheck failed for "
         + "allocateBlock");
     blockManager.allocateBlock(DEFAULT_BLOCK_SIZE,
-        type, factor, OzoneConsts.OZONE, new ExcludeList());
+        replicationConfig, OzoneConsts.OZONE, new ExcludeList());
   }
 
   @Test
   public void testAllocateBlockSucInSafeMode() throws Exception {
     // Test2: Exit safe mode and then try allocateBock again.
     Assert.assertNotNull(blockManager.allocateBlock(DEFAULT_BLOCK_SIZE,
-        type, factor, OzoneConsts.OZONE, new ExcludeList()));
+        replicationConfig, OzoneConsts.OZONE, new ExcludeList()));
   }
 
   @Test(timeout = 10000)
   public void testMultipleBlockAllocation()
       throws IOException, TimeoutException, InterruptedException {
 
-    pipelineManager.createPipeline(type, factor);
-    pipelineManager.createPipeline(type, factor);
+    pipelineManager.createPipeline(replicationConfig);
+    pipelineManager.createPipeline(replicationConfig);
     TestUtils.openAllRatisPipelines(pipelineManager);
 
     AllocatedBlock allocatedBlock = blockManager
-        .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+        .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig, OzoneConsts.OZONE,
             new ExcludeList());
     // block should be allocated in different pipelines
     GenericTestUtils.waitFor(() -> {
       try {
         AllocatedBlock block = blockManager
-            .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+            .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
+                OzoneConsts.OZONE,
                 new ExcludeList());
         return !block.getPipeline().getId()
             .equals(allocatedBlock.getPipeline().getId());
@@ -487,7 +520,8 @@ public class TestBlockManager {
   private boolean verifyNumberOfContainersInPipelines(
       int numContainersPerPipeline) {
     try {
-      for (Pipeline pipeline : pipelineManager.getPipelines(type, factor)) {
+      for (Pipeline pipeline : pipelineManager
+          .getPipelines(replicationConfig)) {
         if (pipelineManager.getNumberOfContainers(pipeline.getId())
             != numContainersPerPipeline) {
           return false;
@@ -507,8 +541,8 @@ public class TestBlockManager {
     // create pipelines
     for (int i = 0;
          i < nodeManager.getNodes(NodeStatus.inServiceHealthy()).size()
-             / factor.getNumber(); i++) {
-      pipelineManager.createPipeline(type, factor);
+             / replicationConfig.getRequiredNodes(); i++) {
+      pipelineManager.createPipeline(replicationConfig);
     }
     TestUtils.openAllRatisPipelines(pipelineManager);
 
@@ -518,7 +552,8 @@ public class TestBlockManager {
     GenericTestUtils.waitFor(() -> {
       try {
         blockManager
-            .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+            .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
+                OzoneConsts.OZONE,
                 new ExcludeList());
       } catch (IOException e) {
       }
@@ -527,7 +562,7 @@ public class TestBlockManager {
     }, 10, 1000);
 
     // close all the containers in all the pipelines
-    for (Pipeline pipeline : pipelineManager.getPipelines(type, factor)) {
+    for (Pipeline pipeline : pipelineManager.getPipelines(replicationConfig)) {
       for (ContainerID cid : pipelineManager
           .getContainersInPipeline(pipeline.getId())) {
         eventQueue.fireEvent(SCMEvents.CLOSE_CONTAINER, cid);
@@ -542,7 +577,8 @@ public class TestBlockManager {
     GenericTestUtils.waitFor(() -> {
       try {
         blockManager
-            .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+            .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig,
+                OzoneConsts.OZONE,
                 new ExcludeList());
       } catch (IOException e) {
       }
@@ -555,11 +591,12 @@ public class TestBlockManager {
   public void testBlockAllocationWithNoAvailablePipelines()
       throws IOException, TimeoutException, InterruptedException {
     for (Pipeline pipeline : pipelineManager.getPipelines()) {
-      pipelineManager.finalizeAndDestroyPipeline(pipeline, false);
+      pipelineManager.closePipeline(pipeline, false);
     }
-    Assert.assertEquals(0, pipelineManager.getPipelines(type, factor).size());
+    Assert.assertEquals(0,
+        pipelineManager.getPipelines(replicationConfig).size());
     Assert.assertNotNull(blockManager
-        .allocateBlock(DEFAULT_BLOCK_SIZE, type, factor, OzoneConsts.OZONE,
+        .allocateBlock(DEFAULT_BLOCK_SIZE, replicationConfig, OzoneConsts.OZONE,
             new ExcludeList()));
   }
 
