@@ -49,12 +49,14 @@ import org.apache.hadoop.net.NetUtils;
 import org.apache.hadoop.ozone.OmUtils;
 import org.apache.hadoop.ozone.OzoneConfigKeys;
 import org.apache.hadoop.ozone.OzoneConsts;
+import org.apache.hadoop.ozone.om.exceptions.OMException;
+import org.apache.hadoop.ozone.ha.ConfUtils;
 import org.apache.hadoop.ozone.om.exceptions.OMLeaderNotReadyException;
 import org.apache.hadoop.ozone.om.exceptions.OMNotLeaderException;
 import org.apache.hadoop.ozone.om.protocolPB.OzoneManagerProtocolClientSideTranslatorPB;
-import org.apache.hadoop.ozone.om.protocolPB.OzoneManagerProtocolPB;
 import org.apache.hadoop.security.UserGroupInformation;
 
+import org.apache.ratis.protocol.exceptions.StateMachineException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -65,27 +67,27 @@ import static org.apache.hadoop.ozone.om.OMConfigKeys.OZONE_OM_ADDRESS_KEY;
  * multiple OMs to connect to. In case of OM failover, client can try
  * connecting to another OM node from the list of proxies.
  */
-public class OMFailoverProxyProvider implements
-    FailoverProxyProvider<OzoneManagerProtocolPB>, Closeable {
+public class OMFailoverProxyProvider<T> implements
+    FailoverProxyProvider<T>, Closeable {
 
   public static final Logger LOG =
       LoggerFactory.getLogger(OMFailoverProxyProvider.class);
 
+  private final String omServiceId;
+  private final ConfigurationSource conf;
+  private final Class<T> protocolClass;
+  private final long omVersion;
+  private final UserGroupInformation ugi;
+  private final Text delegationTokenService;
+
   // Map of OMNodeID to its proxy
-  private Map<String, ProxyInfo<OzoneManagerProtocolPB>> omProxies;
+  private Map<String, ProxyInfo<T>> omProxies;
   private Map<String, OMProxyInfo> omProxyInfos;
   private List<String> omNodeIDList;
 
   private String currentProxyOMNodeId;
   private int currentProxyIndex;
 
-  private final ConfigurationSource conf;
-  private final long omVersion;
-  private final UserGroupInformation ugi;
-  private final Text delegationTokenService;
-
-  private final String omServiceId;
-
   private List<String> retryExceptions = new ArrayList<>();
 
   // OMFailoverProxyProvider, on encountering certain exception, tries each OM
@@ -100,11 +102,13 @@ public class OMFailoverProxyProvider implements
   private Set<String> accessControlExceptionOMs = new HashSet<>();
 
   public OMFailoverProxyProvider(ConfigurationSource configuration,
-      UserGroupInformation ugi, String omServiceId) throws IOException {
+      UserGroupInformation ugi, String omServiceId, Class<T> protocol)
+      throws IOException {
     this.conf = configuration;
-    this.omVersion = RPC.getProtocolVersion(OzoneManagerProtocolPB.class);
+    this.omVersion = RPC.getProtocolVersion(protocol);
     this.ugi = ugi;
     this.omServiceId = omServiceId;
+    this.protocolClass = protocol;
     loadOMClientConfigs(conf, this.omServiceId);
     this.delegationTokenService = computeDelegationTokenService();
 
@@ -129,7 +133,7 @@ public class OMFailoverProxyProvider implements
 
       for (String nodeId : OmUtils.emptyAsSingletonNull(omNodeIds)) {
 
-        String rpcAddrKey = OmUtils.addKeySuffixes(OZONE_OM_ADDRESS_KEY,
+        String rpcAddrKey = ConfUtils.addKeySuffixes(OZONE_OM_ADDRESS_KEY,
             serviceId, nodeId);
         String rpcAddrStr = OmUtils.getOmRpcAddress(config, rpcAddrKey);
         if (rpcAddrStr == null) {
@@ -170,20 +174,18 @@ public class OMFailoverProxyProvider implements
     return currentProxyOMNodeId;
   }
 
-  private OzoneManagerProtocolPB createOMProxy(InetSocketAddress omAddress)
-      throws IOException {
+  private T createOMProxy(InetSocketAddress omAddress) throws IOException {
     Configuration hadoopConf =
         LegacyHadoopConfigurationSource.asHadoopConfiguration(conf);
-    RPC.setProtocolEngine(hadoopConf, OzoneManagerProtocolPB.class,
-        ProtobufRpcEngine.class);
+    RPC.setProtocolEngine(hadoopConf, protocolClass, ProtobufRpcEngine.class);
 
     // FailoverOnNetworkException ensures that the IPC layer does not attempt
     // retries on the same OM in case of connection exception. This retry
     // policy essentially results in TRY_ONCE_THEN_FAIL.
     RetryPolicy connectionRetryPolicy = RetryPolicies
         .failoverOnNetworkException(0);
-    
-    return RPC.getProtocolProxy(OzoneManagerProtocolPB.class, omVersion,
+
+    return (T) RPC.getProtocolProxy(protocolClass, omVersion,
         omAddress, ugi, hadoopConf, NetUtils.getDefaultSocketFactory(
             hadoopConf), (int) OmUtils.getOMClientRpcTimeOut(conf),
         connectionRetryPolicy).getProxy();
@@ -196,7 +198,7 @@ public class OMFailoverProxyProvider implements
    * @return the OM proxy object to invoke methods upon
    */
   @Override
-  public synchronized ProxyInfo getProxy() {
+  public synchronized ProxyInfo<T> getProxy() {
     ProxyInfo currentProxyInfo = omProxies.get(currentProxyOMNodeId);
     if (currentProxyInfo == null) {
       currentProxyInfo = createOMProxy(currentProxyOMNodeId);
@@ -212,7 +214,7 @@ public class OMFailoverProxyProvider implements
     InetSocketAddress address = omProxyInfo.getAddress();
     ProxyInfo proxyInfo;
     try {
-      OzoneManagerProtocolPB proxy = createOMProxy(address);
+      T proxy = createOMProxy(address);
       // Create proxyInfo here, to make it work with all Hadoop versions.
       proxyInfo = new ProxyInfo<>(proxy, omProxyInfo.toString());
       omProxies.put(nodeId, proxyInfo);
@@ -234,17 +236,18 @@ public class OMFailoverProxyProvider implements
           int failovers, boolean isIdempotentOrAtMostOnce)
           throws Exception {
 
+        String omNodeId = getCurrentProxyOMNodeId();
+
         if (LOG.isDebugEnabled()) {
           if (exception.getCause() != null) {
-            LOG.debug("RetryProxy: OM {}: {}: {}", getCurrentProxyOMNodeId(),
+            LOG.debug("RetryProxy: OM {}: {}: {}", omNodeId,
                 exception.getCause().getClass().getSimpleName(),
                 exception.getCause().getMessage());
           } else {
-            LOG.debug("RetryProxy: OM {}: {}", getCurrentProxyOMNodeId(),
+            LOG.debug("RetryProxy: OM {}: {}", omNodeId,
                 exception.getMessage());
           }
         }
-        retryExceptions.add(getExceptionMsg(exception, failovers));
 
         if (exception instanceof ServiceException) {
           OMNotLeaderException notLeaderException =
@@ -266,7 +269,7 @@ public class OMFailoverProxyProvider implements
             // Retry on same OM again as leader OM is not ready.
             // Failing over to same OM so that wait time between retries is
             // incremented
-            performFailoverIfRequired(getCurrentProxyOMNodeId());
+            performFailoverIfRequired(omNodeId);
             return getRetryAction(RetryDecision.FAILOVER_AND_RETRY, failovers);
           }
         }
@@ -286,15 +289,8 @@ public class OMFailoverProxyProvider implements
         if (failovers < maxFailovers) {
           return new RetryAction(fallbackAction, getWaitTime());
         } else {
-          StringBuilder allRetryExceptions = new StringBuilder();
-          allRetryExceptions.append("\n");
-          retryExceptions.stream().forEach(e -> allRetryExceptions.append(e)
-              .append("\n"));
-          LOG.error("Failed to connect to OMs: {}. Attempted {} failovers. " +
-                  "Got following exceptions during retries: {}",
-              getOMProxyInfos(), maxFailovers,
-              allRetryExceptions.toString());
-          retryExceptions.clear();
+          LOG.error("Failed to connect to OMs: {}. Attempted {} failovers.",
+              getOMProxyInfos(), maxFailovers);
           return RetryAction.FAIL;
         }
       }
@@ -333,8 +329,8 @@ public class OMFailoverProxyProvider implements
   }
 
   @Override
-  public Class<OzoneManagerProtocolPB> getInterface() {
-    return OzoneManagerProtocolPB.class;
+  public Class<T> getInterface() {
+    return protocolClass;
   }
 
   /**
@@ -355,7 +351,7 @@ public class OMFailoverProxyProvider implements
    * failover again.
    */
   @Override
-  public void performFailover(OzoneManagerProtocolPB currentProxy) {
+  public void performFailover(T currentProxy) {
     if (LOG.isDebugEnabled()) {
       int currentIndex = getCurrentProxyIndex();
       LOG.debug("Failing over OM proxy to index: {}, nodeId: {}",
@@ -468,6 +464,16 @@ public class OMFailoverProxyProvider implements
           return false;
         }
       }
+    } else if (ex instanceof StateMachineException) {
+      StateMachineException smEx = (StateMachineException) ex;
+      Throwable cause = smEx.getCause();
+      if (cause instanceof OMException) {
+        OMException omEx = (OMException) cause;
+        // Do not failover if the operation was blocked because the OM was
+        // prepared.
+        return omEx.getResult() !=
+            OMException.ResultCodes.NOT_SUPPORTED_OPERATION_WHEN_PREPARED;
+      }
     }
     return true;
   }
@@ -478,7 +484,7 @@ public class OMFailoverProxyProvider implements
    */
   @Override
   public synchronized void close() throws IOException {
-    for (ProxyInfo<OzoneManagerProtocolPB> proxyInfo : omProxies.values()) {
+    for (ProxyInfo<T> proxyInfo : omProxies.values()) {
       if (proxyInfo != null) {
         RPC.stopProxy(proxyInfo.proxy);
       }
@@ -491,7 +497,7 @@ public class OMFailoverProxyProvider implements
   }
 
   @VisibleForTesting
-  public Map<String, ProxyInfo<OzoneManagerProtocolPB>> getOMProxyMap() {
+  public Map<String, ProxyInfo<T>> getOMProxyMap() {
     return omProxies;
   }
 
@@ -500,30 +506,13 @@ public class OMFailoverProxyProvider implements
     return new ArrayList<OMProxyInfo>(omProxyInfos.values());
   }
 
-  private static String getExceptionMsg(Exception e, int retryAttempt) {
-    StringBuilder exceptionMsg = new StringBuilder()
-        .append("Retry Attempt ")
-        .append(retryAttempt)
-        .append(" Exception - ");
-    if (e.getCause() == null) {
-      exceptionMsg.append(e.getClass().getCanonicalName())
-          .append(": ")
-          .append(e.getMessage());
-    } else {
-      exceptionMsg.append(e.getCause().getClass().getCanonicalName())
-          .append(": ")
-          .append(e.getCause().getMessage());
-    }
-    return exceptionMsg.toString();
-  }
-
   /**
    * Check if exception is OMLeaderNotReadyException.
    *
    * @param exception
    * @return OMLeaderNotReadyException
    */
-  private static OMLeaderNotReadyException getLeaderNotReadyException(
+  public static OMLeaderNotReadyException getLeaderNotReadyException(
       Exception exception) {
     Throwable cause = exception.getCause();
     if (cause instanceof RemoteException) {
@@ -556,7 +545,7 @@ public class OMFailoverProxyProvider implements
 
   @VisibleForTesting
   protected void setProxiesForTesting(
-      Map<String, ProxyInfo<OzoneManagerProtocolPB>> testOMProxies,
+      Map<String, ProxyInfo<T>> testOMProxies,
       Map<String, OMProxyInfo> testOMProxyInfos,
       List<String> testOMNodeIDList) {
     this.omProxies = testOMProxies;
