@@ -16,8 +16,7 @@
  */
 package org.apache.hadoop.ozone.om.helpers;
 
-import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationFactor;
-import org.apache.hadoop.hdds.protocol.proto.HddsProtos.ReplicationType;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.MultipartKeyInfo;
 import org.apache.hadoop.ozone.protocol.proto.OzoneManagerProtocolProtos.PartKeyInfo;
 
@@ -32,26 +31,70 @@ import java.util.TreeMap;
 public class OmMultipartKeyInfo extends WithObjectID {
   private final String uploadID;
   private final long creationTime;
-  private final ReplicationType replicationType;
-  private final ReplicationFactor replicationFactor;
+  private final ReplicationConfig replicationConfig;
   private TreeMap<Integer, PartKeyInfo> partKeyInfoList;
 
+  /**
+   * A pointer to parent directory used for path traversal. ParentID will be
+   * used only when the multipart key is created into a FileSystemOptimized(FSO)
+   * bucket.
+   * <p>
+   * For example, if a key "a/b/multiKey1" created into a FSOBucket then each
+   * path component will be assigned an ObjectId and linked to its parent path
+   * component using parent's objectID.
+   * <p>
+   * Say, Bucket's ObjectID = 512, which is the parent for its immediate child
+   * element.
+   * <p>
+   * ------------------------------------------|
+   * PathComponent |   ObjectID   |   ParentID |
+   * ------------------------------------------|
+   *      a        |     1024     |     512    |
+   * ------------------------------------------|
+   *      b        |     1025     |     1024   |
+   * ------------------------------------------|
+   *   multiKey1   |     1026     |     1025   |
+   * ------------------------------------------|
+   */
+  private long parentID;
+
   /**
    * Construct OmMultipartKeyInfo object which holds multipart upload
    * information for a key.
    */
   public OmMultipartKeyInfo(String id, long creationTime,
-      ReplicationType replicationType, ReplicationFactor replicationFactor,
+      ReplicationConfig replicationConfig,
       Map<Integer, PartKeyInfo> list, long objectID, long updateID) {
     this.uploadID = id;
     this.creationTime = creationTime;
-    this.replicationType = replicationType;
-    this.replicationFactor = replicationFactor;
+    this.replicationConfig = replicationConfig;
     this.partKeyInfoList = new TreeMap<>(list);
     this.objectID = objectID;
     this.updateID = updateID;
   }
 
+  /**
+   * Construct OmMultipartKeyInfo object which holds multipart upload
+   * information for a key.
+   */
+  @SuppressWarnings("parameternumber")
+  public OmMultipartKeyInfo(String id, long creationTime,
+      ReplicationConfig replicationConfig,
+      Map<Integer, PartKeyInfo> list, long objectID, long updateID,
+      long parentObjId) {
+    this(id, creationTime, replicationConfig, list, objectID, updateID);
+    this.parentID = parentObjId;
+  }
+
+  /**
+   * Returns parentID.
+   *
+   * @return long
+   */
+  public long getParentID() {
+    return parentID;
+  }
+
   /**
    * Returns the uploadID for this multi part upload of a key.
    * @return uploadID
@@ -76,12 +119,8 @@ public class OmMultipartKeyInfo extends WithObjectID {
     return partKeyInfoList.get(partNumber);
   }
 
-  public ReplicationType getReplicationType() {
-    return replicationType;
-  }
-
-  public ReplicationFactor getReplicationFactor() {
-    return replicationFactor;
+  public ReplicationConfig getReplicationConfig() {
+    return replicationConfig;
   }
 
   /**
@@ -90,11 +129,11 @@ public class OmMultipartKeyInfo extends WithObjectID {
   public static class Builder {
     private String uploadID;
     private long creationTime;
-    private ReplicationType replicationType;
-    private ReplicationFactor replicationFactor;
+    private ReplicationConfig replicationConfig;
     private TreeMap<Integer, PartKeyInfo> partKeyInfoList;
     private long objectID;
     private long updateID;
+    private long parentID;
 
     public Builder() {
       this.partKeyInfoList = new TreeMap<>();
@@ -110,13 +149,8 @@ public class OmMultipartKeyInfo extends WithObjectID {
       return this;
     }
 
-    public Builder setReplicationType(ReplicationType replType) {
-      this.replicationType = replType;
-      return this;
-    }
-
-    public Builder setReplicationFactor(ReplicationFactor replFactor) {
-      this.replicationFactor = replFactor;
+    public Builder setReplicationConfig(ReplicationConfig replConfig) {
+      this.replicationConfig = replConfig;
       return this;
     }
 
@@ -144,9 +178,14 @@ public class OmMultipartKeyInfo extends WithObjectID {
       return this;
     }
 
+    public Builder setParentID(long parentObjId) {
+      this.parentID = parentObjId;
+      return this;
+    }
+
     public OmMultipartKeyInfo build() {
-      return new OmMultipartKeyInfo(uploadID, creationTime, replicationType,
-          replicationFactor, partKeyInfoList, objectID, updateID);
+      return new OmMultipartKeyInfo(uploadID, creationTime, replicationConfig,
+              partKeyInfoList, objectID, updateID, parentID);
     }
   }
 
@@ -160,10 +199,16 @@ public class OmMultipartKeyInfo extends WithObjectID {
     Map<Integer, PartKeyInfo> list = new HashMap<>();
     multipartKeyInfo.getPartKeyInfoListList().forEach(partKeyInfo ->
         list.put(partKeyInfo.getPartNumber(), partKeyInfo));
+
+    final ReplicationConfig replicationConfig = ReplicationConfig
+            .fromTypeAndFactor(
+                    multipartKeyInfo.getType(),
+                    multipartKeyInfo.getFactor());
+
     return new OmMultipartKeyInfo(multipartKeyInfo.getUploadID(),
-        multipartKeyInfo.getCreationTime(), multipartKeyInfo.getType(),
-        multipartKeyInfo.getFactor(), list, multipartKeyInfo.getObjectID(),
-        multipartKeyInfo.getUpdateID());
+        multipartKeyInfo.getCreationTime(), replicationConfig,
+        list, multipartKeyInfo.getObjectID(),
+        multipartKeyInfo.getUpdateID(), multipartKeyInfo.getParentID());
   }
 
   /**
@@ -174,10 +219,11 @@ public class OmMultipartKeyInfo extends WithObjectID {
     MultipartKeyInfo.Builder builder = MultipartKeyInfo.newBuilder()
         .setUploadID(uploadID)
         .setCreationTime(creationTime)
-        .setType(replicationType)
-        .setFactor(replicationFactor)
+        .setType(replicationConfig.getReplicationType())
+        .setFactor(ReplicationConfig.getLegacyFactor(replicationConfig))
         .setObjectID(objectID)
-        .setUpdateID(updateID);
+        .setUpdateID(updateID)
+        .setParentID(parentID);
     partKeyInfoList.forEach((key, value) -> builder.addPartKeyInfoList(value));
     return builder.build();
   }
@@ -204,8 +250,8 @@ public class OmMultipartKeyInfo extends WithObjectID {
   public OmMultipartKeyInfo copyObject() {
     // For partKeyInfoList we can do shallow copy here, as the PartKeyInfo is
     // immutable here.
-    return new OmMultipartKeyInfo(uploadID, creationTime, replicationType,
-        replicationFactor, new TreeMap<>(partKeyInfoList), objectID, updateID);
+    return new OmMultipartKeyInfo(uploadID, creationTime, replicationConfig,
+            new TreeMap<>(partKeyInfoList), objectID, updateID, parentID);
   }
 
 }
