@@ -16,6 +16,6 @@
 DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
 cd "$DIR/../../.." || exit 1
 
-export MAVEN_OPTS="-Xmx4096m"
+export MAVEN_OPTS="-Xmx4096m $MAVEN_OPTS"
 mvn -V -B -Dmaven.javadoc.skip=true -DskipTests clean install "$@"
 exit $?
