@@ -19,6 +19,7 @@ package org.apache.hadoop.ozone.client.rpc;
 
 import java.io.File;
 import java.io.IOException;
+import java.net.URI;
 import java.nio.charset.StandardCharsets;
 import java.security.NoSuchAlgorithmException;
 import java.time.Instant;
@@ -31,12 +32,14 @@ import java.util.Random;
 import java.util.TreeMap;
 import java.util.UUID;
 
+import com.google.common.cache.Cache;
 import org.apache.hadoop.conf.StorageUnit;
 import org.apache.hadoop.crypto.key.KeyProvider;
 import org.apache.hadoop.crypto.key.kms.KMSClientProvider;
 import org.apache.hadoop.crypto.key.kms.server.MiniKMS;
 import org.apache.hadoop.fs.CommonConfigurationKeysPublic;
 import org.apache.hadoop.hdds.HddsConfigKeys;
+import org.apache.hadoop.hdds.client.ReplicationConfig;
 import org.apache.hadoop.hdds.client.ReplicationFactor;
 import org.apache.hadoop.hdds.client.ReplicationType;
 import org.apache.hadoop.hdds.conf.OzoneConfiguration;
@@ -51,7 +54,6 @@ import org.apache.hadoop.ozone.client.ObjectStore;
 import org.apache.hadoop.ozone.client.OzoneBucket;
 import org.apache.hadoop.ozone.client.OzoneClient;
 import org.apache.hadoop.ozone.client.OzoneClientFactory;
-import org.apache.hadoop.ozone.client.OzoneKey;
 import org.apache.hadoop.ozone.client.OzoneKeyDetails;
 import org.apache.hadoop.ozone.client.OzoneVolume;
 import org.apache.hadoop.ozone.client.io.MultipartCryptoKeyInputStream;
@@ -66,7 +68,7 @@ import org.apache.hadoop.ozone.om.helpers.OmMultipartCommitUploadPartInfo;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartInfo;
 import org.apache.hadoop.ozone.om.helpers.OmMultipartUploadCompleteInfo;
 import org.apache.hadoop.ozone.om.helpers.RepeatedOmKeyInfo;
-import org.apache.hadoop.test.GenericTestUtils;
+import org.apache.ozone.test.GenericTestUtils;
 
 import static org.apache.hadoop.hdds.HddsConfigKeys.OZONE_METADATA_DIRS;
 import static org.apache.hadoop.hdds.client.ReplicationFactor.ONE;
@@ -76,6 +78,7 @@ import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.BeforeClass;
 import org.junit.Test;
+import org.mockito.Mockito;
 
 /**
  * This class is to test all the public facing APIs of Ozone Client.
@@ -91,6 +94,7 @@ public class TestOzoneAtRestEncryption {
       storageContainerLocationClient;
 
   private static final String SCM_ID = UUID.randomUUID().toString();
+  private static final String CLUSTER_ID = UUID.randomUUID().toString();
   private static File testDir;
   private static OzoneConfiguration conf;
   private static final String TEST_KEY = "key1";
@@ -126,6 +130,7 @@ public class TestOzoneAtRestEncryption {
     cluster = MiniOzoneCluster.newBuilder(conf)
         .setNumDatanodes(10)
         .setScmId(SCM_ID)
+        .setClusterId(CLUSTER_ID)
         .setBlockSize(BLOCK_SIZE)
         .setChunkSize(CHUNK_SIZE)
         .setStreamBufferSizeUnit(StorageUnit.BYTES)
@@ -145,7 +150,7 @@ public class TestOzoneAtRestEncryption {
     TestOzoneRpcClient.setStorageContainerLocationClient(
         storageContainerLocationClient);
     TestOzoneRpcClient.setStore(store);
-    TestOzoneRpcClient.setScmId(SCM_ID);
+    TestOzoneRpcClient.setClusterId(CLUSTER_ID);
 
     // create test key
     createKey(TEST_KEY, cluster.getOzoneManager().getKmsProvider(), conf);
@@ -177,9 +182,7 @@ public class TestOzoneAtRestEncryption {
   public void testPutKeyWithEncryption() throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    Instant testStartTime = Instant.now();
 
-    String value = "sample value";
     store.createVolume(volumeName);
     OzoneVolume volume = store.getVolume(volumeName);
     BucketArgs bucketArgs = BucketArgs.newBuilder()
@@ -187,35 +190,82 @@ public class TestOzoneAtRestEncryption {
     volume.createBucket(bucketName, bucketArgs);
     OzoneBucket bucket = volume.getBucket(bucketName);
 
-    for (int i = 0; i < 1; i++) {
-      String keyName = UUID.randomUUID().toString();
+    createAndVerifyKeyData(bucket);
+  }
 
-      try (OzoneOutputStream out = bucket.createKey(keyName,
-          value.getBytes(StandardCharsets.UTF_8).length,
-          ReplicationType.STAND_ALONE,
-          ReplicationFactor.ONE, new HashMap<>())) {
-        out.write(value.getBytes(StandardCharsets.UTF_8));
-      }
+  @Test
+  public void testLinkEncryptedBuckets() throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
 
-      OzoneKey key = bucket.getKey(keyName);
-      Assert.assertEquals(keyName, key.getName());
-      byte[] fileContent;
-      int len = 0;
+    // Create source volume/bucket.
+    createVolumeAndBucket(volumeName, bucketName);
 
-      try(OzoneInputStream is = bucket.readKey(keyName)) {
-        fileContent = new byte[value.getBytes(StandardCharsets.UTF_8).length];
-        len = is.read(fileContent);
-      }
+    // Create link volume/bucket.
+    String linkVolumeName = UUID.randomUUID().toString();
+    String linkBucketName = UUID.randomUUID().toString();
+    OzoneBucket linkBucket = createLinkVolumeAndBucket(volumeName, bucketName,
+        linkVolumeName, linkBucketName);
 
-      Assert.assertEquals(len, value.length());
-      Assert.assertTrue(verifyRatisReplication(volumeName, bucketName,
-          keyName, ReplicationType.STAND_ALONE,
-          ReplicationFactor.ONE));
-      Assert.assertEquals(value, new String(fileContent,
-          StandardCharsets.UTF_8));
-      Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
-      Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
+    createAndVerifyKeyData(linkBucket);
+  }
+
+
+  private void createAndVerifyKeyData(OzoneBucket bucket) throws Exception {
+    Instant testStartTime = Instant.now();
+    String keyName = UUID.randomUUID().toString();
+    String value = "sample value";
+    try (OzoneOutputStream out = bucket.createKey(keyName,
+        value.getBytes(StandardCharsets.UTF_8).length,
+        ReplicationType.STAND_ALONE,
+        ReplicationFactor.ONE, new HashMap<>())) {
+      out.write(value.getBytes(StandardCharsets.UTF_8));
     }
+
+    // Verify content.
+    OzoneKeyDetails key = bucket.getKey(keyName);
+    Assert.assertEquals(keyName, key.getName());
+
+    // Check file encryption info is set,
+    // if set key will use this encryption info and encrypt data.
+    Assert.assertTrue(key.getFileEncryptionInfo() != null);
+
+    byte[] fileContent;
+    int len = 0;
+
+    try(OzoneInputStream is = bucket.readKey(keyName)) {
+      fileContent = new byte[value.getBytes(StandardCharsets.UTF_8).length];
+      len = is.read(fileContent);
+    }
+
+
+    Assert.assertEquals(len, value.length());
+    Assert.assertTrue(verifyRatisReplication(bucket.getVolumeName(),
+        bucket.getName(), keyName, ReplicationType.STAND_ALONE,
+        ReplicationFactor.ONE));
+    Assert.assertEquals(value, new String(fileContent, StandardCharsets.UTF_8));
+    Assert.assertFalse(key.getCreationTime().isBefore(testStartTime));
+    Assert.assertFalse(key.getModificationTime().isBefore(testStartTime));
+  }
+
+  private OzoneBucket createVolumeAndBucket(String volumeName,
+      String bucketName) throws Exception{
+    store.createVolume(volumeName);
+    OzoneVolume volume = store.getVolume(volumeName);
+    BucketArgs bucketArgs = BucketArgs.newBuilder()
+        .setBucketEncryptionKey(TEST_KEY).build();
+    volume.createBucket(bucketName, bucketArgs);
+    return volume.getBucket(bucketName);
+  }
+
+  private OzoneBucket createLinkVolumeAndBucket(String sourceVol,
+      String sourceBucket, String linkVol, String linkBucket) throws Exception {
+    store.createVolume(linkVol);
+    OzoneVolume linkVolume = store.getVolume(linkVol);
+    BucketArgs linkBucketArgs = BucketArgs.newBuilder()
+        .setSourceVolume(sourceVol).setSourceBucket(sourceBucket).build();
+    linkVolume.createBucket(linkBucket, linkBucketArgs);
+    return linkVolume.getBucket(linkBucket);
   }
 
   /**
@@ -322,7 +372,8 @@ public class TestOzoneAtRestEncryption {
         keyInfo.getLatestVersionLocations().getLocationList()) {
       ContainerInfo container =
           storageContainerLocationClient.getContainer(info.getContainerID());
-      if (!container.getReplicationFactor().equals(replicationFactor) || (
+      if (!ReplicationConfig.getLegacyFactor(container.getReplicationConfig())
+          .equals(replicationFactor) || (
           container.getReplicationType() != replicationType)) {
         return false;
       }
@@ -347,25 +398,37 @@ public class TestOzoneAtRestEncryption {
 
   @Test
   public void testMPUwithOnePart() throws Exception {
-    testMultipartUploadWithEncryption(1);
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName);
+    testMultipartUploadWithEncryption(bucket, 1);
   }
 
   @Test
   public void testMPUwithTwoParts() throws Exception {
-    testMultipartUploadWithEncryption(2);
-  }
-
-  public void testMultipartUploadWithEncryption(int numParts) throws Exception {
     String volumeName = UUID.randomUUID().toString();
     String bucketName = UUID.randomUUID().toString();
-    String keyName = "mpu_test_key_" + numParts;
+    OzoneBucket bucket = createVolumeAndBucket(volumeName, bucketName);
+    testMultipartUploadWithEncryption(bucket, 2);
+  }
 
-    store.createVolume(volumeName);
-    OzoneVolume volume = store.getVolume(volumeName);
-    BucketArgs bucketArgs = BucketArgs.newBuilder()
-        .setBucketEncryptionKey(TEST_KEY).build();
-    volume.createBucket(bucketName, bucketArgs);
-    OzoneBucket bucket = volume.getBucket(bucketName);
+
+  @Test
+  public void testMPUwithLinkBucket() throws Exception {
+    String volumeName = UUID.randomUUID().toString();
+    String bucketName = UUID.randomUUID().toString();
+    createVolumeAndBucket(volumeName, bucketName);
+
+    String linkVolumeName = UUID.randomUUID().toString();
+    String linkBucketName = UUID.randomUUID().toString();
+    OzoneBucket linkBucket = createLinkVolumeAndBucket(volumeName, bucketName,
+        linkVolumeName, linkBucketName);
+    testMultipartUploadWithEncryption(linkBucket, 2);
+  }
+
+  public void testMultipartUploadWithEncryption(OzoneBucket bucket,
+      int numParts) throws Exception {
+    String keyName = "mpu_test_key_" + numParts;
 
     // Initiate multipart upload
     String uploadID = initiateMultipartUpload(bucket, keyName, STAND_ALONE,
@@ -491,4 +554,26 @@ public class TestOzoneAtRestEncryption {
         offset + " and length " + readData.length,
         inputDataForComparison, readData);
   }
+
+  @Test
+  public void testGetKeyProvider() throws Exception {
+    KeyProvider kp1 = store.getKeyProvider();
+    KeyProvider kpSpy = Mockito.spy(kp1);
+    Assert.assertNotEquals(kpSpy, kp1);
+    Cache<URI, KeyProvider> cacheSpy =
+        ((RpcClient)store.getClientProxy()).getKeyProviderCache();
+    cacheSpy.put(store.getKeyProviderUri(), kpSpy);
+    KeyProvider kp2 = store.getKeyProvider();
+    Assert.assertEquals(kpSpy, kp2);
+
+    // Verify the spied key provider is closed upon ozone client close
+    ozClient.close();
+    Mockito.verify(kpSpy).close();
+
+    KeyProvider kp3 = ozClient.getObjectStore().getKeyProvider();
+    Assert.assertNotEquals(kp3, kpSpy);
+    // Restore ozClient and store
+    TestOzoneRpcClient.setOzClient(OzoneClientFactory.getRpcClient(conf));
+    TestOzoneRpcClient.setStore(ozClient.getObjectStore());
+  }
 }
