@@ -85,7 +85,7 @@ public class TestOzoneContainer {
       Mockito.when(dsm.getDatanodeDetails()).thenReturn(datanodeDetails);
       Mockito.when(context.getParent()).thenReturn(dsm);
       container = new OzoneContainer(datanodeDetails, conf, context, null);
-      //Set scmId and manually start ozone container.
+      //Set clusterId and manually start ozone container.
       container.start(UUID.randomUUID().toString());
 
       XceiverClientGrpc client = new XceiverClientGrpc(pipeline, conf);
@@ -128,10 +128,10 @@ public class TestOzoneContainer {
       container = new OzoneContainer(datanodeDetails, conf,
           context, null);
 
-      String scmId = UUID.randomUUID().toString();
-      container.start(scmId);
+      String clusterId = UUID.randomUUID().toString();
+      container.start(clusterId);
       try {
-        container.start(scmId);
+        container.start(clusterId);
       } catch (Exception e) {
         Assert.fail();
       }
