@@ -33,6 +33,7 @@ import org.apache.hadoop.ozone.om.exceptions.OMException;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.ratis.utils.OzoneManagerDoubleBufferHelper;
 import org.apache.hadoop.ozone.om.request.OMClientRequest;
+import org.apache.hadoop.ozone.om.response.bucket.acl.OMBucketAclResponse;
 import org.apache.hadoop.ozone.util.BooleanBiFunction;
 import org.apache.hadoop.ozone.om.request.util.ObjectParser;
 import org.apache.hadoop.ozone.om.response.OMClientResponse;
@@ -204,8 +205,11 @@ public abstract class OMBucketAclRequest extends OMClientRequest {
    * @param exception
    * @return OMClientResponse
    */
-  abstract OMClientResponse onFailure(OMResponse.Builder omResponse,
-      IOException exception);
+  OMClientResponse onFailure(OMResponse.Builder omResponse,
+      IOException exception) {
+    return new OMBucketAclResponse(
+        createErrorOMResponse(omResponse, exception));
+  }
 
   /**
    * Completion hook for final processing before return without lock.
