@@ -25,7 +25,7 @@ import org.apache.hadoop.hdds.utils.db.cache.CacheValue;
 import org.apache.hadoop.ozone.om.helpers.OmBucketInfo;
 import org.apache.hadoop.ozone.om.helpers.OmKeyInfo;
 import org.apache.hadoop.ozone.om.helpers.OmVolumeArgs;
-import org.apache.hadoop.ozone.om.ratis.OMTransactionInfo;
+import org.apache.hadoop.hdds.utils.TransactionInfo;
 import org.apache.hadoop.ozone.om.request.TestOMRequestUtils;
 import org.junit.Assert;
 import org.junit.Before;
@@ -33,6 +33,7 @@ import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
 
+import java.util.Arrays;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
@@ -68,22 +69,22 @@ public class TestOmMetadataManager {
   @Test
   public void testTransactionTable() throws Exception {
     omMetadataManager.getTransactionInfoTable().put(TRANSACTION_INFO_KEY,
-        new OMTransactionInfo.Builder().setCurrentTerm(1)
+        new TransactionInfo.Builder().setCurrentTerm(1)
             .setTransactionIndex(100).build());
 
     omMetadataManager.getTransactionInfoTable().put(TRANSACTION_INFO_KEY,
-        new OMTransactionInfo.Builder().setCurrentTerm(2)
+        new TransactionInfo.Builder().setCurrentTerm(2)
             .setTransactionIndex(200).build());
 
     omMetadataManager.getTransactionInfoTable().put(TRANSACTION_INFO_KEY,
-        new OMTransactionInfo.Builder().setCurrentTerm(3)
+        new TransactionInfo.Builder().setCurrentTerm(3)
             .setTransactionIndex(250).build());
 
-    OMTransactionInfo omTransactionInfo =
+    TransactionInfo transactionInfo =
         omMetadataManager.getTransactionInfoTable().get(TRANSACTION_INFO_KEY);
 
-    Assert.assertEquals(3, omTransactionInfo.getTerm());
-    Assert.assertEquals(250, omTransactionInfo.getTransactionIndex());
+    Assert.assertEquals(3, transactionInfo.getTerm());
+    Assert.assertEquals(250, transactionInfo.getTransactionIndex());
 
 
   }
@@ -623,4 +624,12 @@ public class TestOmMetadataManager {
     }
   }
 
+  @Test
+  public void testAllTablesAreProperInOMMetadataManagerImpl() {
+    Set<String> tablesByDefinition =
+        new HashSet<>(Arrays.asList(OmMetadataManagerImpl.ALL_TABLES));
+    Set<String> tablesInManager = omMetadataManager.listTableNames();
+
+    Assert.assertEquals(tablesByDefinition, tablesInManager);
+  }
 }
\ No newline at end of file
