@@ -30,25 +30,27 @@ import java.util.UUID;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.stream.Collectors;
 
+import org.apache.hadoop.conf.Configuration;
 import com.google.common.annotations.VisibleForTesting;
-import org.apache.hadoop.hdds.conf.ConfigurationSource;
 import org.apache.hadoop.hdds.protocol.DatanodeDetails;
 import org.apache.hadoop.hdds.protocol.proto.HddsProtos;
 import org.apache.hadoop.hdds.protocol.proto.StorageContainerDatanodeProtocolProtos.ContainerReplicaProto;
 import org.apache.hadoop.hdds.scm.container.ContainerID;
 import org.apache.hadoop.hdds.scm.container.ContainerInfo;
+import org.apache.hadoop.hdds.scm.container.ContainerManagerImpl;
 import org.apache.hadoop.hdds.scm.container.ContainerNotFoundException;
 import org.apache.hadoop.hdds.scm.container.ContainerReplica;
 import org.apache.hadoop.hdds.scm.container.ContainerReplicaNotFoundException;
-import org.apache.hadoop.hdds.scm.container.SCMContainerManager;
 import org.apache.hadoop.hdds.scm.container.common.helpers.ContainerWithPipeline;
+import org.apache.hadoop.hdds.scm.ha.SCMHAManager;
+import org.apache.hadoop.hdds.scm.ha.SequenceIdGenerator;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineID;
 import org.apache.hadoop.hdds.scm.pipeline.PipelineManager;
 import org.apache.hadoop.hdds.utils.db.DBStore;
 import org.apache.hadoop.hdds.utils.db.Table;
 import org.apache.hadoop.ozone.recon.persistence.ContainerHistory;
 import org.apache.hadoop.ozone.recon.persistence.ContainerHealthSchemaManager;
-import org.apache.hadoop.ozone.recon.spi.ContainerDBServiceProvider;
+import org.apache.hadoop.ozone.recon.spi.ReconContainerMetadataManager;
 import org.apache.hadoop.ozone.recon.spi.StorageContainerServiceProvider;
 
 import org.slf4j.Logger;
@@ -57,16 +59,19 @@ import org.slf4j.LoggerFactory;
 /**
  * Recon's overriding implementation of SCM's Container Manager.
  */
-public class ReconContainerManager extends SCMContainerManager {
+public class ReconContainerManager extends ContainerManagerImpl {
 
   private static final Logger LOG =
       LoggerFactory.getLogger(ReconContainerManager.class);
-  private final StorageContainerServiceProvider scmClient;
+  private StorageContainerServiceProvider scmClient;
+  private PipelineManager pipelineManager;
   private final ContainerHealthSchemaManager containerHealthSchemaManager;
-  private final ContainerDBServiceProvider cdbServiceProvider;
+  private final ReconContainerMetadataManager cdbServiceProvider;
   private final Table<UUID, DatanodeDetails> nodeDB;
   // Container ID -> Datanode UUID -> Timestamp
   private final Map<Long, Map<UUID, ContainerReplicaHistory>> replicaHistoryMap;
+  // Pipeline -> # of open containers
+  private final Map<PipelineID, Integer> pipelineToOpenContainer;
 
   /**
    * Constructs a mapping class that creates mapping between container names
@@ -78,22 +83,27 @@ public class ReconContainerManager extends SCMContainerManager {
    *
    * @throws IOException on Failure.
    */
+  @SuppressWarnings("parameternumber")
   public ReconContainerManager(
-      ConfigurationSource conf,
+      Configuration conf,
+      DBStore store,
       Table<ContainerID, ContainerInfo> containerStore,
-      DBStore batchHandler,
       PipelineManager pipelineManager,
       StorageContainerServiceProvider scm,
       ContainerHealthSchemaManager containerHealthSchemaManager,
-      ContainerDBServiceProvider containerDBServiceProvider)
+      ReconContainerMetadataManager reconContainerMetadataManager,
+      SCMHAManager scmhaManager,
+      SequenceIdGenerator sequenceIdGen)
       throws IOException {
-    super(conf, containerStore, batchHandler, pipelineManager);
+    super(conf, scmhaManager, sequenceIdGen, pipelineManager, containerStore);
     this.scmClient = scm;
+    this.pipelineManager = pipelineManager;
     this.containerHealthSchemaManager = containerHealthSchemaManager;
-    this.cdbServiceProvider = containerDBServiceProvider;
+    this.cdbServiceProvider = reconContainerMetadataManager;
     // batchHandler = scmDBStore
-    this.nodeDB = ReconSCMDBDefinition.NODES.getTable(batchHandler);
+    this.nodeDB = ReconSCMDBDefinition.NODES.getTable(store);
     this.replicaHistoryMap = new ConcurrentHashMap<>();
+    this.pipelineToOpenContainer = new ConcurrentHashMap<>();
   }
 
   /**
@@ -106,32 +116,106 @@ public class ReconContainerManager extends SCMContainerManager {
   public void checkAndAddNewContainer(ContainerID containerID,
       ContainerReplicaProto.State replicaState,
       DatanodeDetails datanodeDetails)
-      throws IOException {
-    if (!exists(containerID)) {
+      throws Exception {
+    if (!containerExist(containerID)) {
       LOG.info("New container {} got from {}.", containerID,
           datanodeDetails.getHostName());
       ContainerWithPipeline containerWithPipeline =
           scmClient.getContainerWithPipeline(containerID.getId());
       LOG.debug("Verified new container from SCM {}, {} ",
           containerID, containerWithPipeline.getPipeline().getId());
-      // If no other client added this, go ahead and add this container.
-      if (!exists(containerID)) {
-        addNewContainer(containerID.getId(), containerWithPipeline);
-      }
+      // no need call "containerExist" to check, because
+      // 1 containerExist and addNewContainer can not be atomic
+      // 2 addNewContainer will double check the existence
+      addNewContainer(containerWithPipeline);
     } else {
-      // Check if container state is not open. In SCM, container state
-      // changes to CLOSING first, and then the close command is pushed down
-      // to Datanodes. Recon 'learns' this from DN, and hence replica state
-      // will move container state to 'CLOSING'.
-      ContainerInfo containerInfo = getContainer(containerID);
-      if (containerInfo.getState().equals(HddsProtos.LifeCycleState.OPEN)
-          && !replicaState.equals(ContainerReplicaProto.State.OPEN)
-          && isHealthy(replicaState)) {
-        LOG.info("Container {} has state OPEN, but Replica has State {}.",
-            containerID, replicaState);
-        updateContainerState(containerID, FINALIZE);
+      checkContainerStateAndUpdate(containerID, replicaState);
+    }
+  }
+
+  /**
+   * Check and add new containers in batch if not already present in Recon.
+   *
+   * @param containerReplicaProtoList list of containerReplicaProtos.
+   */
+  public void checkAndAddNewContainerBatch(
+      List<ContainerReplicaProto> containerReplicaProtoList) {
+    Map<Boolean, List<ContainerReplicaProto>> containers =
+        containerReplicaProtoList.parallelStream()
+        .collect(Collectors.groupingBy(c ->
+            containerExist(ContainerID.valueOf(c.getContainerID()))));
+
+    List<ContainerReplicaProto> existContainers = null;
+    if (containers.containsKey(true)) {
+      existContainers = containers.get(true);
+    }
+    List<Long> noExistContainers = null;
+    if (containers.containsKey(false)){
+      noExistContainers = containers.get(false).parallelStream().
+          map(ContainerReplicaProto::getContainerID)
+          .collect(Collectors.toList());
+    }
+
+    if (null != noExistContainers) {
+      List<ContainerWithPipeline> verifiedContainerPipeline =
+          scmClient.getExistContainerWithPipelinesInBatch(noExistContainers);
+      LOG.debug("{} new containers have been verified by SCM , " +
+              "{} containers not found at SCM",
+          verifiedContainerPipeline.size(),
+          noExistContainers.size() - verifiedContainerPipeline.size());
+      for (ContainerWithPipeline cwp : verifiedContainerPipeline) {
+        try {
+          addNewContainer(cwp);
+        } catch (IOException ioe) {
+          LOG.error("Exception while checking and adding new container.", ioe);
+        }
       }
     }
+
+    if (null != existContainers) {
+      for (ContainerReplicaProto crp : existContainers) {
+        ContainerID cID = ContainerID.valueOf(crp.getContainerID());
+        ContainerReplicaProto.State crpState = crp.getState();
+        try {
+          checkContainerStateAndUpdate(cID, crpState);
+        } catch (Exception ioe){
+          LOG.error("Exception while " +
+              "checkContainerStateAndUpdate container", ioe);
+        }
+      }
+    }
+  }
+
+  /**
+   *  Check if container state is not open. In SCM, container state
+   *  changes to CLOSING first, and then the close command is pushed down
+   *  to Datanodes. Recon 'learns' this from DN, and hence replica state
+   *  will move container state to 'CLOSING'.
+   *
+   * @param containerID containerID to check
+   * @param state  state to be compared
+   * @throws IOException
+   */
+
+  private void checkContainerStateAndUpdate(ContainerID containerID,
+                                            ContainerReplicaProto.State state)
+      throws Exception {
+    ContainerInfo containerInfo = getContainer(containerID);
+    if (containerInfo.getState().equals(HddsProtos.LifeCycleState.OPEN)
+        && !state.equals(ContainerReplicaProto.State.OPEN)
+        && isHealthy(state)) {
+      LOG.info("Container {} has state OPEN, but given state is {}.",
+          containerID, state);
+      final PipelineID pipelineID = containerInfo.getPipelineID();
+      // subtract open container count from the map
+      int curCnt = pipelineToOpenContainer.getOrDefault(pipelineID, 0);
+      if (curCnt == 1) {
+        pipelineToOpenContainer.remove(pipelineID);
+      } else if (curCnt > 0) {
+        pipelineToOpenContainer.put(pipelineID, curCnt - 1);
+      }
+      updateContainerState(containerID, FINALIZE);
+    }
   }
 
   private boolean isHealthy(ContainerReplicaProto.State replicaState) {
@@ -142,24 +226,25 @@ public class ReconContainerManager extends SCMContainerManager {
 
   /**
    * Adds a new container to Recon's container manager.
-   * @param containerId id
    * @param containerWithPipeline containerInfo with pipeline info
    * @throws IOException on Error.
    */
-  public void addNewContainer(long containerId,
-                              ContainerWithPipeline containerWithPipeline)
+  public void addNewContainer(ContainerWithPipeline containerWithPipeline)
       throws IOException {
     ContainerInfo containerInfo = containerWithPipeline.getContainerInfo();
-    getLock().lock();
     try {
-      boolean success = false;
       if (containerInfo.getState().equals(HddsProtos.LifeCycleState.OPEN)) {
         PipelineID pipelineID = containerWithPipeline.getPipeline().getId();
-        if (getPipelineManager().containsPipeline(pipelineID)) {
-          getContainerStateManager().addContainerInfo(containerId,
-              containerInfo, getPipelineManager(),
-              containerWithPipeline.getPipeline());
-          success = true;
+        if (pipelineManager.containsPipeline(pipelineID)) {
+          getContainerStateManager().addContainer(containerInfo.getProtobuf());
+          pipelineManager.addContainerToPipeline(
+              containerWithPipeline.getPipeline().getId(),
+              containerInfo.containerID());
+          // update open container count on all datanodes on this pipeline
+          pipelineToOpenContainer.put(pipelineID,
+                    pipelineToOpenContainer.getOrDefault(pipelineID, 0) + 1);
+          LOG.info("Successfully added container {} to Recon.",
+              containerInfo.containerID());
         } else {
           // Get open container for a pipeline that Recon does not know
           // about yet. Cannot update internal state until pipeline is synced.
@@ -168,26 +253,17 @@ public class ReconContainerManager extends SCMContainerManager {
               pipelineID, containerInfo.containerID()));
         }
       } else {
-        // Non 'Open' Container. No need to worry about pipeline since SCM
-        // returns a random pipelineID.
-        getContainerStateManager().addContainerInfo(containerId,
-            containerInfo, getPipelineManager(), null);
-        success = true;
-      }
-      if (success) {
-        addContainerToDB(containerInfo);
-        LOG.info("Successfully added container {} to Recon.",
+        getContainerStateManager().addContainer(containerInfo.getProtobuf());
+        LOG.info("Successfully added no open container {} to Recon.",
             containerInfo.containerID());
       }
     } catch (IOException ex) {
       LOG.info("Exception while adding container {} .",
           containerInfo.containerID(), ex);
-      getPipelineManager().removeContainerFromPipeline(
+      pipelineManager.removeContainerFromPipeline(
           containerInfo.getPipelineID(),
-          new ContainerID(containerInfo.getContainerID()));
+          ContainerID.valueOf(containerInfo.getContainerID()));
       throw ex;
-    } finally {
-      getLock().unlock();
     }
   }
 
@@ -371,4 +447,8 @@ public class ReconContainerManager extends SCMContainerManager {
     return nodeDB;
   }
 
+  public Map<PipelineID, Integer> getPipelineToOpenContainer() {
+    return pipelineToOpenContainer;
+  }
+
 }
